// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(IW_GAME_ACTIONS_H_)
#define IW_GAME_ACTIONS_H_

#include "s3e.h"

#include "IwGameUtil.h"
#include "IwGameXoml.h"


//
//
//
//
//  CIwGameEvent - Repersents some named event with an actions list
//
//
//
//
struct CIwGameEvent
{
	unsigned int		EventName;		// Event method type (e.g. OnResume, OnTapped)
	unsigned int		ActionsName;	// Name of Actions collection to perform when event occurs
};

//
//
//
//
//  CIwGameEventManager - Manages a collection of events
//
//
//
//
class CIwGameEventManager
{
public:
	// Public access to event iteration
	typedef CIwList<CIwGameEvent*>::iterator _Iterator;
	_Iterator					begin() { return Events.begin(); }
	_Iterator					end() { return Events.end(); }
protected:
	CIwList<CIwGameEvent*> Events;
public:
	CIwGameEventManager() {}
	virtual ~CIwGameEventManager()
	{
		for (_Iterator it = begin(); it != end(); ++it)
		{
			delete *it;
		}
		Events.clear();
	}

	void	addEvent(const char* event_name, const char* actions_name)
	{
		CIwGameEvent* evt = new CIwGameEvent();
		evt->EventName = CIwGameString::CalculateHash(event_name);
		evt->ActionsName = CIwGameString::CalculateHash(actions_name);
		Events.push_back(evt);
	}

	void	addEvent(CIwGameEvent* evt)
	{
		Events.push_back(evt);
	}

	CIwGameEvent* findEvent(unsigned int event_name_hash)
	{
		for (_Iterator it = begin(); it != end(); ++it)
		{
			if ((*it)->EventName == event_name_hash)
			{
				return *it;
			}
		}

		return NULL;
	}
};

//
//
//
//
//  CIwGameAction - An action is a command that can be excuted from XOML
//
//
//
//
struct CIwGameAction
{
	unsigned int		MethodName;		// Action method type
	CIwGameString		Parameter1;		// Action parameter1
	CIwGameString		Parameter2;		// Action parameter2
};

//
//
//
//
//  CIwGameActions - A collection of actions
//
//
//
//
class CIwGameActions : public IIwGameXomlResource
{
public:
	// Public access to actions iteration
	typedef CIwList<CIwGameAction*>::iterator _Iterator;
	_Iterator					begin() { return Actions.begin(); }
	_Iterator					end() { return Actions.end(); }
	// Properties
protected:
	CIwList<CIwGameAction*>		Actions;			// List of actions
public:
	// Properties end

	CIwGameActions() : IIwGameXomlResource() { setClassType("actions");	}
	virtual ~CIwGameActions()
	{
		for (_Iterator it = begin(); it != end(); ++it)
		{
			delete *it;
		}
		Actions.clear();
	}

	void			Execute(IIwGameXomlResource* source);

	// Implementation of IIwGameXomlResource interface
	bool			LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node);
};

//
//
//
//
//  CIwGameActionsreator - Creates an instance of an actions ovject
//
//
//
//
class CIwGameActionsCreator : public IIwGameXomlClassCreator
{
public:
	CIwGameActionsCreator()
	{
		setClassName("actions");
	}
	IIwGameXomlResource* CreateInstance(IIwGameXomlResource* parent) { return new CIwGameActions(); }
};



#endif // IW_GAME_ACTIONS_H_