// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(_IW_GAME_XOML_H_)
#define _IW_GAME_XOML_H_


#include "s3efile.h"
#include "IwGameString.h"
#include "IwGameUtil.h"
#include "IwGameXml.h"

class CIwGame;
class CIwGameStyle;
struct CIwGameAction;

// 
// 
//
//
// CIwGameXomlNames - pre-hashed strings of classes, attributes and certain attributes data used to speed up string searchimng
// 
// 
//
//
struct CIwGameXomlNames
{
public:
	static unsigned int Game_Hash;
	static unsigned int Scene_Hash;
	static unsigned int Resource_Hash;
	static unsigned int Actor_Hash;
	static unsigned int ActorImage_Hash;
	static unsigned int ResourceGroup_Hash;
	static unsigned int Animation_Hash;
	static unsigned int Box2dMaterial_Hash;
	static unsigned int Shape_Hash;
	static unsigned int Actions_Hash;
	static unsigned int Camera_Hash;
	static unsigned int Image_Hash;
	static unsigned int Timeline_Hash;
	static unsigned int Style_Hash;
	static unsigned int Variable_Hash;

	static unsigned int Action_Hash;
	static unsigned int Name_Hash;
	static unsigned int File_Hash;
	static unsigned int Current_Hash;
	static unsigned int CanvasSize_Hash;
	static unsigned int FixAspect_Hash;
	static unsigned int LockWidth_Hash;
	static unsigned int Extents_Hash;
	static unsigned int AllowSuspend_Hash;
	static unsigned int Clipping_Hash;
	static unsigned int Active_Hash;
	static unsigned int Visible_Hash;
	static unsigned int Layers_Hash;
	static unsigned int Colliders_Hash;
	static unsigned int Type_Hash;
	static unsigned int Position_Hash;
	static unsigned int Velocity_Hash;
	static unsigned int VelocityDamping_Hash;
	static unsigned int Angle_Hash;
	static unsigned int AngularVelocity_Hash;
	static unsigned int AngularVelocityDamping_Hash;
	static unsigned int Scale_Hash;
	static unsigned int Color_Hash;
	static unsigned int Colour_Hash;
	static unsigned int Layer_Hash;
	static unsigned int Collidable_Hash;
	static unsigned int CollisionSize_Hash;
	static unsigned int CollisionRect_Hash;
	static unsigned int WrapPosition_Hash;
	static unsigned int Size_Hash;
	static unsigned int GroupFile_Hash;
	static unsigned int Frame_Hash;
	static unsigned int Value_Hash;
	static unsigned int Time_Hash;
	static unsigned int Repeat_Hash;
	static unsigned int Duration_Hash;
	static unsigned int SrcRect_Hash;
	static unsigned int Location_Hash;
	static unsigned int Preload_Hash;
	static unsigned int Blocking_Hash;
	static unsigned int AutoPlay_Hash;
	static unsigned int Anim_Hash;
	static unsigned int StartAtTime_Hash;
	static unsigned int Target_Hash;
	static unsigned int Width_Hash;
	static unsigned int Height_Hash;
	static unsigned int Radius_Hash;
	static unsigned int Points_Hash;
	static unsigned int Point_Hash;
	static unsigned int Data_Hash;
	static unsigned int Density_Hash;
	static unsigned int Friction_Hash;
	static unsigned int Restitution_Hash;
	static unsigned int COM_Hash;
	static unsigned int Gravity_Hash;
	static unsigned int WorldScale_Hash;
	static unsigned int Sensor_Hash;
	static unsigned int CollisionFlags_Hash;
	static unsigned int Set_Hash;
	static unsigned int Property_Hash;
	static unsigned int HitTest_Hash;
	static unsigned int FlipX_Hash;
	static unsigned int FlipY_Hash;
	static unsigned int Method_Hash;
	static unsigned int Param1_Hash;
	static unsigned int Param2_Hash;
	static unsigned int OnTapped_Hash;
	static unsigned int OnBeginTouch_Hash;
	static unsigned int OnEndTouch_Hash;
	static unsigned int OnSuspend_Hash;
	static unsigned int OnResume_Hash;

	static unsigned int ChangeScene_Hash;
	static unsigned int SuspendScene_Hash;
	static unsigned int ResumeScene_Hash;
	static unsigned int HideScene_Hash;
	static unsigned int ShowScene_Hash;
	static unsigned int ActivateScene_Hash;
	static unsigned int DeactivateScene_Hash;
	static unsigned int KillScene_Hash;
	static unsigned int HideActor_Hash;
	static unsigned int ShowActor_Hash;
	static unsigned int ActivateActor_Hash;
	static unsigned int DeactivateActor_Hash;
	static unsigned int KillActor_Hash;
	static unsigned int PlayTimeline_Hash;
	static unsigned int StopTimeline_Hash;
	static unsigned int SetTimeline_Hash;
	static unsigned int PlaySound_Hash;
	static unsigned int PlayMusic_Hash;
	static unsigned int StopMusic_Hash;
	static unsigned int SetVar_Hash;

	static unsigned int bool_Hash;
	static unsigned int float_Hash;
	static unsigned int int_Hash;
	static unsigned int vec2_Hash;
	static unsigned int vec3_Hash;
	static unsigned int vec4_Hash;
	static unsigned int rect_Hash;
	static unsigned int string_Hash;
	static unsigned int box_Hash;
	static unsigned int circle_Hash;
	static unsigned int polygon_Hash;
	static unsigned int static_Hash;
	static unsigned int dynamic_Hash;
	static unsigned int kinematic_Hash;
};

// 
// 
//
//
// IIwGameXomlResource - Base class that is used by classes that can be instanced from XOML markup
// 
// To create a class that can be instantiated from XOML you should:
// 1. Derive the class from CIwGameXomlClass
// 2. Set its XOML base type (game, actor, scene, animation etc..)
// 3. Implement a class derived from IIwGameXomlClassCreator that implements CreateInstance() which creates an instance of the class
// 4. Add the class type to the list of class creators in IW_GAME_XOML using addClass()
//
//
//
//
class IIwGameXomlResource
{
	// Properties
protected:
#if defined (_DEBUG)
	CIwGameString			Name;					// Resource name
	CIwGameString			ClassType;				// Resource class type
#endif	// _DEBUG
	bool					Managed;				// True is managed by the resource system. Managed resources are auto clean ed up by the resource system
	unsigned int			NameHash;				// Resource name hash
	unsigned int			ClassTypeHash;			// Base type of XOML class (e.g. Scene, Actor, Animation, Game etc..)
public:
	void					setName(const char* name)
	{
#if defined (_DEBUG)
		Name = name;
#endif	// _DEBUG
		NameHash = CIwGameString::CalculateHash(name);
	}
	unsigned int			getNameHash()							{ return NameHash; }
	void					setClassType(const char* type_name)
	{
#if defined (_DEBUG)
		ClassType = type_name;
#endif	// _DEBUG
		ClassTypeHash = CIwGameString::CalculateHash(type_name);
	}
	unsigned int			getClassTypeHash()						{ return ClassTypeHash; }
	void					setManaged(bool managed)				{ Managed = managed; }
	bool					isManaged() const						{ return Managed; }
	// Properties end
public:
	IIwGameXomlResource() : Managed(false) {}
	virtual ~IIwGameXomlResource() {}

	virtual bool LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node) = 0;
};

// 
// 
//
//
// IIwGameXomlClassCreator - Base class that is used by classes that create an instance of a XOML resource type
//
//
//
//
class IIwGameXomlClassCreator
{
	// Properties
protected:
#if defined(_DEBUG)
	CIwGameString			ClassName;				// Name of class
#endif
	unsigned int			ClassNameHash;			// Hashed name of class
public:
	void					setClassName(const char* name)
	{
#if defined(_DEBUG)
		ClassName = name;
#endif
		ClassNameHash = CIwGameString::CalculateHash(name);
	}
	unsigned int			getClassNameHash() const							{ return ClassNameHash; }
	// Properties end
public:
	virtual IIwGameXomlResource* CreateInstance(IIwGameXomlResource* parent) = 0;
};

// 
// 
//
//
// CIwGameXomlVariable - A basic XOML variable
//
//
//
//
enum eCIwGameXomlVariableType
{
	VT_String, 
	VT_Bool, 
	VT_Float, 
	VT_Int, 
	VT_Vec2, 
	VT_Vec3, 
	VT_Vec4, 
};

struct CIwGameXomlVariable
{
	// Properties
protected:
	CIwGameString				Value;				// Value of variable
public:
	virtual void				setValue(const char* value)	{ Value = value; }
	CIwGameString&				getValue()			{ return Value; Modified = false; }
	// Properties end
	CIwGameString				Name;				// Name of variable
	eCIwGameXomlVariableType	Type;				// Type of variable
	bool						Modified;			// Modified since last read flag
	CIwGameXomlVariable() : Type(VT_String), Modified(false) {}
};
struct CIwGameXomlVariableBool : CIwGameXomlVariable
{
	bool						NativeValue;		// Native value of the variable
	CIwGameXomlVariableBool() : CIwGameXomlVariable() { Type = VT_Bool; }
	void						setValue(const char* value)
	{
		Value = value;
		NativeValue = Value.GetAsBool();
		Modified = true;
	}
};
struct CIwGameXomlVariableFloat : CIwGameXomlVariable
{
	float						NativeValue;		// Native value of the variable
	CIwGameXomlVariableFloat() : CIwGameXomlVariable()  { Type = VT_Float; }
	void						setValue(const char* value)
	{
		Value = value;
		NativeValue = Value.GetAsFloat();
		Modified = true;
	}
};
struct CIwGameXomlVariableInt : CIwGameXomlVariable
{
	int							NativeValue;		// Native value of the variable
	CIwGameXomlVariableInt() : CIwGameXomlVariable()  { Type = VT_Int; }
	void						setValue(const char* value)
	{
		Value = value;
		NativeValue = Value.GetAsInt();
		Modified = true;
	}
};
struct CIwGameXomlVariableVec2 : CIwGameXomlVariable
{
	CIwFVec2					NativeValue;		// Native value of the variable
	CIwGameXomlVariableVec2() : CIwGameXomlVariable()  { Type = VT_Vec2; }
	void						setValue(const char* value)
	{
		Value = value;
		float components[16];
		if (Value.GetAsListOfFloat(components) == 2)
		{
			NativeValue.x = components[0];
			NativeValue.y = components[1];
			Modified = true;
		}
	}
};
struct CIwGameXomlVariableVec3 : CIwGameXomlVariable
{
	CIwFVec3					NativeValue;		// Native value of the variable
	CIwGameXomlVariableVec3() : CIwGameXomlVariable()  { Type = VT_Vec3; }
	void						setValue(const char* value)
	{
		Value = value;
		float components[16];
		if (Value.GetAsListOfFloat(components) == 3)
		{
			NativeValue.x = components[0];
			NativeValue.y = components[1];
			NativeValue.z = components[2];
			Modified = true;
		}
	}
};
struct CIwGameXomlVariableVec4 : CIwGameXomlVariable
{
	CIwFVec4					NativeValue;		// Native value of the variable
	CIwGameXomlVariableVec4() : CIwGameXomlVariable()  { Type = VT_Vec4; }
	void						setValue(const char* value)
	{
		Value = value;
		float components[16];
		if (Value.GetAsListOfFloat(components) == 4)
		{
			NativeValue.x = components[0];
			NativeValue.y = components[1];
			NativeValue.z = components[2];
			NativeValue.w = components[3];
			Modified = true;
		}
	}
};

// 
// 
//
//
// CIwGameXomlVariableManager - Manages a collection of XOML variables
//
//
//
//
class CIwGameXomlVariableManager
{
public:
	// Public access to iteration
	typedef CIwList<CIwGameXomlVariable*>::iterator _Iterator;
	_Iterator				begin()		{ return Variables.begin(); }
	_Iterator				end()		{ return Variables.end(); }

protected:
	// Properties
	CIwList<CIwGameXomlVariable*>	Variables;			// A collection of xoml variables
public:
	void					addVariable(CIwGameXomlVariable* variable);
	void					addVariable(const char* name, eCIwGameXomlVariableType type = VT_String);
	void					addVariable(const char* name, eCIwGameXomlVariableType type, const char* value);
	void					setVariable(unsigned int name_hash, const char* value);
	void					setVariable(const char* name, const char* value);
	CIwGameXomlVariable*	findVariable(unsigned int name_hash);
	CIwGameXomlVariable*	findVariable(const char* name);
	void					clearVariables();
	// Properties end

public:
	CIwGameXomlVariableManager() {}
	~CIwGameXomlVariableManager() { clearVariables(); }

	CIwGameXomlVariable*	CreateVariable(const char* name, eCIwGameXomlVariableType type = VT_String);
};

// 
// 
//
//
// IIwGameXomlAction - Base class that is used by classes that create an actions that can be executed in XOML
//
//
//
//
class IIwGameXomlAction
{
	// Properties
protected:
#if defined(_DEBUG)
	CIwGameString			ActionName;				// Name of action
#endif
	unsigned int			ActionNameHash;			// Name of action (hashed)
public:
	void					setActionName(const char* name)
	{
#if defined(_DEBUG)
		ActionName = name;
#endif
		ActionNameHash = CIwGameString::CalculateHash(name);
	}
	unsigned int			getActionNameHash() const							{ return ActionNameHash; }
	// Properties end
public:
	virtual void Execute(IIwGameXomlResource* source, CIwGameAction* action) = 0;
};

// 
// 
//
//
// CIwGameXomlResourceManager - CIwGameResourceManager manages a collection of resource groups
// 
// 
// 
//
class CIwGameXomlResourceManager
{
public:
	// Public access to class creator iteration
	typedef CIwList<IIwGameXomlResource*>::iterator _Iterator;
	_Iterator				begin()		{ return Resources.begin(); }
	_Iterator				end()		{ return Resources.end(); }

protected:
	// Properties
	CIwList<IIwGameXomlResource*>	Resources;			// A collection of resources resources
public:
	// Properties end

public:
	CIwGameXomlResourceManager() {}
	~CIwGameXomlResourceManager() { clearResources(); }

	void					addResource(IIwGameXomlResource* resource);
	void					removeResource(IIwGameXomlResource* resource);
	void					removeResource(unsigned int name_hash, unsigned int type_hash);
	IIwGameXomlResource*	findResource(unsigned int name_hash, unsigned int type_hash, bool global_search = true);
	IIwGameXomlResource*	findResource(const char* name, unsigned int type_hash, bool global_search = true);
	IIwGameXomlResource*	findResource(const char* name, const char* type, bool global_search = true);
	void					clearResources();
};



// 
// 
//
//
// CIwGameXoml - CIwGameXoml is the main controller responsible for instantiating objects from XOML based files
// 
// 
// 
//
#define IW_GAME_XOML	CIwGameXoml::getInstance()
class CIwGameXoml
{
public:
	CDEFINE_SINGLETON(CIwGameXoml)

	// Public access to class creator iteration
	typedef CIwList<IIwGameXomlClassCreator*>::iterator _Iterator;
	_Iterator				begin() { return ClassCreators.begin(); }
	_Iterator				end() { return ClassCreators.end(); }
	typedef CIwList<IIwGameXomlAction*>::iterator _ActionsIterator;
	_ActionsIterator		actions_begin() { return Actions.begin(); }
	_ActionsIterator		actions_end() { return Actions.end(); }

protected:
	// Properties
public:
	void			addClass(IIwGameXomlClassCreator* creator)
	{
		ClassCreators.push_back(creator);
	}
	void			addAction(IIwGameXomlAction* action)
	{
		Actions.push_back(action);
	}
	// Properties end

protected:
	CIwList<IIwGameXomlClassCreator*> ClassCreators;
	CIwList<IIwGameXomlAction*> Actions;

public:
	void			Init();
	void			Release();

	bool			Process(IIwGameXomlResource* parent, const char* filename, bool reset_xml_pools = true);	// Pass false if you have other XML files in memory that are in use or they may be written over
	bool			Process(IIwGameXomlResource* parent, CIwGameXmlNode* node);

	// Utility
	static void		ApplyStyle(CIwGameStyle* style, CIwGameXmlNode* node);
};

//
//
//
//
// CIwGameXomlAddVariable utility class to allow XOML files to add variables
// 
//
// 
//
class CIwGameXomlAddVariable : public IIwGameXomlResource
{
public:
protected:
	//// Properties
public:
	//// Properties end
public:
	CIwGameXomlAddVariable() : IIwGameXomlResource() { setClassType("variable"); }
	// Implementation of IIwGameXomlClass interface
	bool				LoadFromXoml(IIwGameXomlResource* parebt, bool load_children, CIwGameXmlNode* node);
};

class CIwGameXomlAddVariableCreator : public IIwGameXomlClassCreator
{
public:
	CIwGameXomlAddVariableCreator()
	{
		setClassName("variable");
	}
	IIwGameXomlResource* CreateInstance(IIwGameXomlResource* parent)	{ return new CIwGameXomlAddVariable(); }
};

//
//
//
//
// CIwGameXomlLoad utility class to allow loading of XOMl files from XOML
// 
//
// 
//
class CIwGameXomlLoad : public IIwGameXomlResource
{
public:
protected:
	//// Properties
public:
	//// Properties end
public:
	CIwGameXomlLoad() : IIwGameXomlResource() { setClassType("loadxoml"); }
	// Implementation of IIwGameXomlClass interface
	bool				LoadFromXoml(IIwGameXomlResource* parebt, bool load_children, CIwGameXmlNode* node);
};

class CIwGameXomlLoadCreator : public IIwGameXomlClassCreator
{
public:
	CIwGameXomlLoadCreator()
	{
		setClassName("loadxoml");
	}
	IIwGameXomlResource* CreateInstance(IIwGameXomlResource* parent)	{ return new CIwGameXomlLoad(); }
};


#endif	// _IW_GAME_XOML_H_
