// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameCamera.h"
#include "IwGameResources.h"
#include "IwGameScene.h"

//
//
//
//
// CIwGameCamera implementation
//
//
//
//

void CIwGameCamera::Update()
{
	if (TransformDirty)
	{
		// Update camera transform
		Transform.SetIdentity();
		Transform.SetRot((iwangle)((Angle * IW_ANGLE_2PI) / 360.0f));
		Transform.ScaleRot((iwfixed)(Scale * IW_GEOM_ONE));
		TransformDirty = false;
	}
}

bool CIwGameCamera::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	// Process Actor attributes
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
		{
			setName((*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::Position_Hash)
		{
			if (!(*it)->GetValueAsPoint(Position))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for Camera::Position");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::Angle_Hash)
		{
			Angle = (*it)->GetValueAsFloat();
		}
		else
		if (name_hash == CIwGameXomlNames::Scale_Hash)
		{
			Scale = (*it)->GetValueAsFloat();
		}
		else
		if (name_hash == CIwGameXomlNames::Target_Hash)
		{
			setTarget((*it)->GetValue().c_str());
		}
	}

	// If we are declared inside a scene then shape is local to the scene
	if (parent != NULL && parent->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
	{
		((CIwGameScene*)parent)->getResourceManager()->addResource(this);
	}
	else
	{
		IW_GAME_GLOBAL_RESOURCES->getResourceManager()->addResource(this);
	}

	return true;
}

